"use client";
import React, { createContext, useEffect, useState } from "react";
import AppKit from "@/context/provider";
import { logConsole } from "@/utils/logConsole";
import { NETWORKS } from "@/configs/networks";
import { useTonWallet } from "@/hooks/useTonWallet";
import { useEvmWallet } from "@/hooks/useEvmWallet";
import { TonConnectUIProvider } from "@tonconnect/ui-react";
import { CreateConnectorFn } from "@wagmi/core";
import { QueryClient, QueryClientProvider } from "@tanstack/react-query";
// @ts-expect-error: Ignore wagmi's error
import { WagmiProvider } from "wagmi";
import { wagmiConfig } from "@/configs/wagmiConfig";

const queryClientOptions = new QueryClient({
  defaultOptions: {
    queries: {
      gcTime: 1_000 * 60 * 60 * 24,
    },
  },
});

export const DashboardContext = createContext<DashboardContextType>({
  tonConnected: false,
  setTonConnected: () => {},
  evmConnected: false,
  setEvmConnected: () => {},
  selectedWallet: {},
  setSelectedWallet: () => {},
  disableTon: false,
  setDisableTon: () => {},
  disbleEvm: false,
  setDisableEvm: () => {},
  processing: false,
  setProcessing: () => {},
  isConnected: false,
  setIsConnected: () => {},
  walletConnected: false,
  setWalletConnected: () => {},
  handleSendTransaction: () => {},
  allowDisconnect: () => {},
});

function AssetChainKit({
  children,
  manifestUrl,
  projectId,
  infuraApiKey,
  metadata,
  defaultConnector,
}: Readonly<{
  children: React.ReactNode;
  manifestUrl: string;
  projectId: string;
  infuraApiKey: string;
  metadata: any;
  defaultConnector: CreateConnectorFn | undefined;
}>) {
  const [tonConnected, setTonConnected] = useState(false);
  const [evmConnected, setEvmConnected] = useState(false);
  const [selectedWallet, setSelectedWallet] = useState<any>(
    NETWORKS.assetchain_mainnet
  );
  const [disableTon, setDisableTon] = useState(false);
  const [disbleEvm, setDisableEvm] = useState(false);
  const [processing, setProcessing] = useState(false);
  const [isConnected, setIsConnected] = useState(false);
  const [walletConnected, setWalletConnected] = useState(false);
  const { disconnectWallet, sendTransaction } = useTonWallet();
  const { disconnectEvmWallet, sendTransactionEvm } = useEvmWallet({
    projectId,
    infuraApiKey,
    metadata,
    defaultConnector,
  });

  const onlyOneWallet = () => {
    if (tonConnected) {
      setDisableEvm(true);
    }
    if (evmConnected) {
      setDisableTon(true);
    }
  };

  const selectWallet = (wallet_: string) => {
    const wallet = wallet_.toLocaleLowerCase();
    logConsole({ wallet });
    if (wallet == "assetchain") {
      setSelectedWallet(NETWORKS.assetchain_mainnet);
    }
    if (wallet == "ton") {
      setSelectedWallet(NETWORKS.ton_mainnet);
    }
  };

  useEffect(() => {
    onlyOneWallet();
  }, [tonConnected, evmConnected]);

  useEffect(() => {
    if (tonConnected || evmConnected) {
      setWalletConnected(true);
      if (tonConnected) {
        selectWallet("ton");
      }
      if (evmConnected) {
        selectWallet("assetchain");
      }
    } else {
      setWalletConnected(false);
    }
    if (selectedWallet) {
      setWalletConnected(true);
    }
  }, [tonConnected, evmConnected, selectedWallet]);

  const allowDisconnect = async () => {
    if (!selectedWallet) return;

    const actions: { [key: string]: () => Promise<void> } = {
      RWA: disconnectEvmWallet,
      TON: disconnectWallet,
    };

    const currency: keyof typeof actions =
      selectedWallet?.currency as keyof typeof actions;

    if (currency && actions[currency]) {
      await actions[currency]();
      setDisableEvm(false);
      setDisableTon(false);
      setWalletConnected(false);
    }
  };

  const handleSendTransaction = async (amount: string, to: string) => {
    const sendAmount = amount;
    const recipientAddress = to;
    try {
      if (selectedWallet) {
        if (tonConnected || evmConnected) {
          if (selectedWallet.currency == "TON") {
            const tx = await sendTransaction({
              value: sendAmount,
              to: recipientAddress,
            });
            return tx;
          }

          if (selectedWallet.currency == "RWA") {
            const tx = await sendTransactionEvm({
              value: sendAmount,
              to: recipientAddress,
            });
            return tx;
          }
        }
      } else {
        console.error("Connect a wallet first");
      }
    } catch (error) {
      logConsole({ error });
    }
  };

  return (
    <WagmiProvider config={wagmiConfig}>
      <QueryClientProvider client={queryClientOptions}>
      <TonConnectUIProvider manifestUrl={manifestUrl}>
        <DashboardContext.Provider
            value={{
              tonConnected,
              setTonConnected,
              evmConnected,
              setEvmConnected,
              selectedWallet,
              setSelectedWallet,
              disableTon,
              setDisableTon,
              disbleEvm,
              setDisableEvm,
              processing,
              setProcessing,
              isConnected,
              setIsConnected,
              walletConnected,
              setWalletConnected,
              handleSendTransaction,
              allowDisconnect,
            }}
          >
            {children}
          </DashboardContext.Provider>
        </TonConnectUIProvider>
      </QueryClientProvider>
    </WagmiProvider>
  );
}

export default AssetChainKit;
